
// C++20 example
// source: https://en.cppreference.com/w/cpp/algorithm/ranges/generate

#include <algorithm>
#include <array>
#include <iostream>
#include <random>
#include <string_view>

// pseudorandom number generator, uniform int range 1..6
int dice() {
    static std::uniform_int_distribution<int> distr{1, 6};
    static std::random_device device;   // true random seed source
#if 1
    static std::mt19937 engine{device()};
#else
    static std::mt19937 engine;         // the same sequence each run
#endif
    return distr(engine);
}

// example of "generate"
void my_iota(auto& v, int n) {
    std::ranges::generate(v, [&n] () { return n++; });
}

// print container
void print(const std::string_view comment, const auto& v) {
    std::cout << comment;
    for (int i : v) { std::cout << i << ' '; }
    std::cout << '\n';
}

int main() {
    std::array<int, 8> v;

#if 1
    my_iota(v, 1);
#else
    std::iota(v.begin(), v.end(), 1);
#endif
    print("iota: ", v);
    std::cout << "\n";

    std::generate(v.begin(), v.end(), dice);
    print("dice: ", v);

    std::ranges::generate(v, dice);     // C++20
    print("dice: ", v);
}

